@@ -250,8 +250,9 @@ def padlist(container, size, default=None):
         ('George', 'Costanza', 'NYC')
         >>> first, last, city = padlist(['George', 'Costanza'], 3)
         ('George', 'Costanza', None)
-        >>> first, last, city, planet = padlist(['George', 'Costanza',
-                                                 'NYC'], 4, default='Earth')
+        >>> first, last, city, planet = padlist(
+        ...     ['George', 'Costanza', 'NYC'], 4, default='Earth',
+        ... )
         ('George', 'Costanza', 'NYC', 'Earth')
 
     """
